// For standard C++ printing (std::cout)
#include "started_pch.h"
#include <iostream>
// For Naming Service capabilities
#include <orbsvcs/CosNamingC.h>

// To obtain handle to distributed Servant
#include "SimpleInterfaceC.h"
// For Multicasting capabilities
#include <tao/Object_T.h>
#include <orbsvcs/PortableGroup/MIOP.h>
int main (int argc, char *argv[])
{
    // Use ACE-specific TRY/CATCH mechanism
    ACE_TRY_NEW_ENV
    {
        // Initialize and error check ORB
        CORBA::ORB_var orb = CORBA::ORB_init (argc, argv);
        if (CORBA::is_nil(orb.in()))
        {
			std::cout << "Could not initialize ORB" << std::endl;
            exit(1);
        }
        CORBA::Object_var NameServiceObjectVar = orb->resolve_initial_references("NameService");
        // Initialize and error check Naming Service objects
        if (CORBA::is_nil(NameServiceObjectVar.in()))
        {
            std::cout << "Could not initialize Naming Service" << std::endl;
            exit(1);
        }
        CosNaming::NamingContext_var NamingContextVar =
                CosNaming::NamingContext::_narrow(NameServiceObjectVar.in());
        if (CORBA::is_nil(NamingContextVar.in()))
        {
            std::cout << "Could not initialize Naming Context" << std::endl;
            exit(1);
        }
        // Obtain and error check two multicast group handles
        CosNaming::Name MCastName1(1);
        MCastName1.length(1);
        MCastName1[0].id = CORBA::string_dup("MCAST1");
        CORBA::Object_var MCastTemp1 = NamingContextVar->resolve(MCastName1);
        if (CORBA::is_nil(MCastTemp1.in()))
        {
            std::cout << "Could not initialize MCastTemp1" << std::endl;
            exit(1);
        }
        SimpleInterface_var MULTI_handle1 =
                TAO::Narrow_Utils<SimpleInterface>::unchecked_narrow(
                        MCastTemp1.in(),
                        _TAO_SimpleInterface_Proxy_Broker_Factory_function_pointer);
        if (CORBA::is_nil (MULTI_handle1.in ()))
        {
            std::cout << "Could not initialize multicast handle 1" <<
                    std::endl;
            exit(1);
        }
        CosNaming::Name McastName2(1);
        McastName2.length(1);
        McastName2[0].id = CORBA::string_dup("MCAST2");
        CORBA::Object_var MCastTemp2 = NamingContextVar->resolve(McastName2);
        if (CORBA::is_nil(MCastTemp2.in()))
        {
            std::cout << "Could not initialize MCastTemp2" << std::endl;
            exit(1);
        }
        SimpleInterface_var MULTI_handle2 =
                TAO::Narrow_Utils<SimpleInterface>::unchecked_narrow(
                        MCastTemp2.in(),
                        _TAO_SimpleInterface_Proxy_Broker_Factory_function_pointer);
        if (CORBA::is_nil(MULTI_handle2.in()))
        {
            std::cout << "Could not initialize multicast handle 2" <<
                    std::endl;
            exit(1);
        }
        MULTI_handle1->send_forty_two(42);
        MULTI_handle2->send_forty_two(42);
        MULTI_handle1->shutdown();
        // Cleanup
        orb->destroy ();
    }
    ACE_CATCHANY
    {
        ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION,
                             "Exception caught:");
        return 1;
    }
    ACE_ENDTRY;
    return 0;
}
